
#include <lubishev/Exception.h>
#include <lubishev/Scene.h>
#include <lubishev/SceneEvent.h>

#include <iostream>

namespace lubishev {

SceneEvent *SceneEvent::mSelected   = 0;
SceneEvent *SceneEvent::mMouseAbove = 0;

SceneEvent::SceneEvent(Scene *scn, Event *e) : mScene(scn),
                                               mEvent(e),
                                               mDisplayAddons(false),
                                               mContainsResult(NONE) {
                                                   /*
    mEvent->tags.push_back("Event");
    mEvent->tags.push_back(QString("%1").arg(e->id));
    */
    mDoc = new TextDocument(QColor(0, 0, 0), QColor(0, 0, 0, 0));
    mDoc->setDocumentMargin(0);
    refreshRect();
    refreshText();
}

SceneEvent::~SceneEvent() {
    delete mDoc;
    if (mMouseAbove == this)
        mMouseAbove = 0;
    if (mSelected == this)
        mSelected = 0;
}

SceneEvent::PART SceneEvent::contains(const QPointF &pos) const {
    if (!mDisplayAddons)
        mContainsResult = mRect.contains(pos) ? EVENT : NONE;
    else {
        if (mRect.contains(pos))
            mContainsResult = EVENT;
        else if (mHandleTop.contains(pos))
            mContainsResult = HANDLE_TOP;
        else if (mHandleBottom.contains(pos))
            mContainsResult = HANDLE_BOTTOM;
        else
            mContainsResult = NONE;
    }
    return mContainsResult;
}

SceneEvent::PART SceneEvent::containsResult() const {
    return mContainsResult;
}

void SceneEvent::dehoverMouseAbove() {
    mMouseAbove = 0;
}

void SceneEvent::deselectSelected() {
    mSelected = 0;
}

void SceneEvent::draw(QPainter *p) {
    QBrush brush = p->brush();
    brush.setStyle(Qt::SolidPattern);
    p->setBrush(brush);
    QColor fillColor(QColor(0, 0, 255, 50));
    QColor durationColor(QColor(0, 0, 255, 100)); // also for addons.
    mDisplayAddons = false;
    if (isMouseAbove() && isSelected()) {
        fillColor = QColor(0, 255, 0, 50);
        durationColor = QColor(255, 0, 0);
        mDisplayAddons = true;
    }
    else if (isMouseAbove()) {
        fillColor = QColor(0, 255, 0, 50);
        durationColor = QColor(0, 255, 0);
        mDisplayAddons = true;
    }
    else if (isSelected()) {
        fillColor = QColor(255, 0, 0, 50);
        durationColor = QColor(255, 0, 0);
        mDisplayAddons = true;
    }
    setColor(*p, fillColor);
    p->drawRect(mRect);
    // Duration.
    setColor(*p, durationColor);
    u32 duration = mEvent->end() - mEvent->start();
    setFontSize(*p, mScene->secondHeight() * (duration > mScene->snap() * 2 ? 760 : 380));
    p->drawText(mRect,
                Qt::AlignRight | Qt::AlignVCenter,
                secondsToString(duration));
    if (mDisplayAddons) {
        durationColor.setAlpha(255);
        setColor(*p, durationColor);
        // Handles.
        float htop    = mScene->timeToY(mEvent->start() - mScene->snap());
        float hbottom = mScene->timeToY(mEvent->end()   + mScene->snap());
        float etop    = mScene->timeToY(mEvent->start());
        float ebottom = mScene->timeToY(mEvent->end());
        mHandleTop = QRectF(QPointF(0, htop), QPointF(mScene->width(), etop));
        mHandleBottom = QRectF(QPointF(0, hbottom),
                               QPointF(mScene->width(), ebottom));
        QBrush brush = p->brush();
        brush.setStyle(Qt::DiagCrossPattern);
        p->setBrush(brush);
        p->drawRect(mHandleTop);
        p->drawRect(mHandleBottom);
        brush.setStyle(Qt::SolidPattern);
        p->setBrush(brush);
        // Interval.
        setFontSize(*p, mScene->secondHeight() * 380);
        p->drawText(mRect.topLeft(), timeToString(mEvent->start()));
        QPointF pos(0, mScene->timeToY(mScene->snap() + mEvent->end()));
        p->drawText(pos, timeToString(mEvent->end()));
    }
    drawText(p);
}

void SceneEvent::drawText(QPainter *p) {
    setFontSize(*p, mScene->secondHeight() * 190);
    mDoc->setDefaultFont(p->font());
    mDoc->setColors(QColor(0, 0, 0, 100), QColor(0, 0, 0, 0));
    if (isMouseAbove() || isSelected())
        mDoc->setColors(QColor(0, 0, 0), QColor(0, 0, 0, 0));
    // Draw.
    p->translate(mRect.x(), mRect.y());
    QRectF clip = mRect;
    clip.translate(-mRect.x(), -mRect.y());
    mDoc->drawContents(p, clip);
    p->translate(-mRect.x(), -mRect.y());
}

Event* SceneEvent::event() {
    return mEvent;
}

bool SceneEvent::isMouseAbove() const {
    return mMouseAbove == this;
}

bool SceneEvent::isSelected() const {
    return mSelected == this;
}

SceneEvent* SceneEvent::mouseAbove() {
    return mMouseAbove;
}

void SceneEvent::refreshRect() {
    mRect =
        QRectF(QPointF(1,                   mScene->timeToY(mEvent->start())),
               QPointF(mScene->width() - 1, mScene->timeToY(mEvent->end())));
    mDoc->setPageSize(QSizeF(mRect.width(), mRect.height()));
}

void SceneEvent::refreshText() {
    QStringList list = mEvent->tags();
    QString s;
    for (s32 i = 0; i < list.size(); i++)
        if (s.length())
            s += "\n" + list[i];
        else
            s += list[i];
    mDoc->clear();
    mDoc->setPlainText(s);
}

SceneEvent* SceneEvent::selected() {
    return mSelected;
}

void SceneEvent::setMouseAbove(bool state) {
    if (state)
        mMouseAbove = this;
    else if (isMouseAbove())
        mMouseAbove = 0;
}

void SceneEvent::setSelected(bool state) {
    if (state)
        mSelected = this;
    else if (isSelected())
        mSelected = 0;
}

void SceneEvent::setSelectedPartTime(time_t t) {
    switch (mContainsResult) {
        case NONE:
            break;
        case EVENT: 
            {
                time_t diff = mEvent->start() - t;
                mEvent->setTimes(mEvent->start() - diff, mEvent->end() - diff);
            }
            break;
        case HANDLE_TOP:
            {
                if (t < mEvent->end())
                    mEvent->setTimes(t, mEvent->end());
                else
                    LUBISHEV_EXCEPTION("Warning",
                                       "Start date becomes older than the end date");
            }
            break;
        case HANDLE_BOTTOM:
            {
                if (t > mEvent->start())
                    mEvent->setTimes(mEvent->start(), t);
                else
                    LUBISHEV_EXCEPTION("Warning",
                                       "Start date becomes older than the end date");
            }
            break;
    }
    refreshRect();
}

} // namespace lubishev

